#!/bin/bash

# Percepio Detect Client for Linux
#
# The Detect Client is an integrated set of developer tools for viewing debugging data
# provided with alerts. This runs on each user’s computer. When clicking on payload links
# in the Dashboard, the Client is notified and loads the selected data in the right tool:
# 
# * Percepio Tracealyzer, shows TraceRecorder traces.
# * Core Dump Viewer, shows core dumps on crashes.
#
# The Client is file-oriented. It receives an "alert key" that works as a relative path 
# in the Alert Directory, where it reads the alert files. It also require some additional
# "context" files to show the payloads, as described below.
#
# The Core Dump Viewer requires at minimum the .elf file. This must be the same version
# as programmed on the device that sent the alert. This can also leverage the source code
# files to display source code snippets at faults, but that is optional. For displaying the
# call stack, with function names etc, only the .elf file is needed.
#
# Tracealyzer does not require .elf files or source code, but can leverage .elf files for
# "compact logging" functions (e.g. xTracePrintCompactF) that allow more efficient tracing.
#
# It is assumed that all data files are accessible via the file system, for example in
# network folders. If the build artifacts are stored in another way, for example in Sonatype
# Nexus or JFrog Artifactory, you can modify userdefinedhooks.py to download the files using
# wget or curl. See userdefinedhooks.py for documentation and a code example.
#
# When alerts come from different builds, the matching .elf file can be resolved by using
# metadata provided with the alert, in particular the Revision and Product fields.
# They are set in the target-side library, DFM, in dfmConfig.h.
#
#
# Specify the settings below to match your project. Otherwise the demo data is used instead. 
#
# For example:
#  ALERTS_DIR="/mnt/server/percepio_detect_alerts"
#
# ALERTS_DIR
# The folder containing the alert data, that is monitored by the Server. 
# In the demo, this is the same local folder as in percepio-server.sh and
# percepio-receiver.sh. In a multi-user setup this is typically a network folder,
# like "/mnt/server/percepio_detect_alerts".
#
# ELF_DIR
# The base directory where the .elf files are found, for example your local 
# build directory or your build archive. If you download the elf file using
# userdefinedhooks.py, this setting is not important but must be a valid
# directory since mounted in the Docker container.
#
# ELF_REL_PATH
# The relative path to the .elf file within ELF_DIR. This is appended to the ELF
# directory to get the full path. This may include ${product} and ${revision} to
# include the corresponding metadata from the selected alert.
#
# Example: ELF_REL_PATH='${product}/builds/image-${revision}.elf'
#
# SRC_DIR
# Provide the base path of your source repository. This will be mounted and
# available inside the client, so GDB can show source code. This is optional
# but must point to an existing directory since mounted in the Docker container.
#
# SRV_REL_PATH
# Provide the relative path to the source directory (relative to SRC_DIR).
# This may contain variables such as ${product} and ${revision} that are
# resolved dynamically for each selected alert, like for ELF_REL_PATH.
#
# Example: SRC_REL_PATH='${product}-${revision}'

# Default configuration (demo data)
DEFAULT_ALERTS_DIR="$(pwd)/../test-data/alert-files"

# Default configuration (demo data)
DEFAULT_ELF_DIR="$(pwd)/../test-data"

# Using alert metadata to find the right ELF file. Single-quotes must be used here.
DEFAULT_ELF_REL_PATH='${revision}.elf'

# Default configuration (demo data)
DEFAULT_SRC_DIR="$(pwd)/../device-integration/demo-projects/"

# Using alert metadata to find the project folder in SRC_DIR. Single-quotes must be used here.
DEFAULT_SRC_REL_PATH='${revision}'


# Image tag
TAG="2025.1"

# Docker image
IMAGE="percepio/percepio-client:$TAG"


if [ "$1" = "cleanup" ]; then
  echo -e "This will delete all Percepio Detect client images and also the volume for the Percepio Detect client. The image will be downloaded again on next start. You will need to re-enter the license key for Percepio Tracealyzer."
  read -p "Do you want to proceed? [yes/no] " confirmation

  if [ "$confirmation" != "yes" ]; then
    echo "Aborted."
    exit 0
  fi

  echo "Deleting all percepio/percepio-client images."
  docker images percepio/percepio-client -q | xargs -r docker rmi

  echo "Deleting docker volume percepio-client-volume."
  docker volume rm percepio-client-volume

  echo "Run ./percepio-client.sh to start the Percepio Detect client."
  exit 0
fi


if [ -z "${ALERTS_DIR}" ]; then
ALERTS_DIR=$DEFAULT_ALERTS_DIR
echo "ALERTS_DIR:   Using default path (demo data)."
fi

if [ -z "${ELF_DIR}" ]; then
ELF_DIR=$DEFAULT_ELF_DIR
echo "ELF_DIR:      Using default path (demo data)."
fi

if [ -z "${ELF_REL_PATH}" ]; then
ELF_REL_PATH=$DEFAULT_ELF_REL_PATH
echo "ELF_REL_PATH: Using default path (demo data)."
fi

if [ -z "${SRC_DIR}" ]; then
SRC_DIR=$DEFAULT_SRC_DIR
echo "SRC_DIR:      Using default path (demo data)."
fi

if [ -z "${SRC_REL_PATH}" ]; then
SRC_REL_PATH=$DEFAULT_SRC_REL_PATH
echo "SRC_REL_PATH: Using default path (demo data)."
fi


# Translates to absolute paths.
ALERTS_DIR=$(realpath $ALERTS_DIR)
ELF_DIR=$(realpath $ELF_DIR)
SRC_DIR=$(realpath $SRC_DIR)


check_path() {
  dir_to_check=$1
  label=$2
  if [ -z "$dir_to_check" ]; then
    echo "Checking $label: Error, parameter $label not set."
    exit 1
  fi  
  if [ ! -d "$dir_to_check" ]; then
    echo "Checking $label: Error, not a valid path."
    echo "$label: $dir_to_check"
    exit 1
  fi 
}

# Verifies paths
check_path $ALERTS_DIR "ALERTS_DIR"
check_path $ELF_DIR "ELF_DIR"
check_path $SRC_DIR "SRC_DIR"

check_path_access_rights() {
  local dir_to_check=$1

  # Check if the directory is writable for others (should not be required for the Client anymore, but possibly for the Server (alert sender)?)
  if [ -d "$dir_to_check" ] && [ -w "$dir_to_check" ] && [ "$(stat -c %A "$dir_to_check" | cut -c 9)" = "w" ]; then
      echo "Alert directory permissions are OK."
  else
      # Print error message in red and exit
      echo -e "\033[1;31mThe directory '$dir_to_check' is NOT writable for other users. Consider changing the permissions using:\033[0m"
      echo -e "\033[1;31mchmod o+rw '$dir_to_check'\033[0m"
      exit 1
  fi
}

# check if ALERTS_DIR is writable for others
check_path_access_rights "$ALERTS_DIR"

myhostname=$(uname -n)

docker run --network=host \
           -u $(id -u):$(id -g) \
           -e USER=$(whoami) \
           -e HOME=/home/app \
           -v percepio-client-volume:/home/app/settings \
           -v $HOME:/host-mounts/home \
           -v $(pwd)/userdefinedhooks.py:/host-mounts/userdefinedhooks.py \
           -v $ALERTS_DIR:/host-mounts/alert_data \
           -v $ELF_DIR:/host-mounts/device_elf \
           -v $SRC_DIR:/host-mounts/device_src \
           --env ELF_REL_PATH=$ELF_REL_PATH \
           --env SRC_REL_PATH=$SRC_REL_PATH \
           -it --shm-size=1g -it --rm -e DISPLAY=$DISPLAY \
           -v /tmp/.X11-unix:/tmp/.X11-unix \
           -v $XAUTHORITY:/home/app/.Xauthority \
           -v /dev/dri:/dev/dri \
           -h $myhostname \
           -e XDG_RUNTIME_DIR=/home/app \
           -e XDG_CACHE_HOME=/tmp \
           $IMAGE \
           $1




