param (
    [Parameter(Mandatory = $true, ValueFromPipeline = $true)]
    [ValidateSet("start", "stop", "logs", "cleanup")]
    [string]$Action
)

# The parameter ALERTS_DIR should specify the folder containing the alert files.
# Note that Percepio Client uses a separate ALERTS_DIR setting that should point
# to the same directory. The included demo data directory is used by default. 
# To use a different ALERTS_DIR setting, uncomment the line below.

# This needs to be an absolute path.
#$ALERTS_DIR = "..."

# Default ALERTS_DIR, containing demo data.
$DEFAULT_ALERTS_DIR = Join-Path (Get-Location).Path "..\test-data\alert-files"

# Specify your Detect Server license key below, e.g. $LICENSE="ABCD-ABCD-ABCD-ABCD"
$LICENSE=""

if ([string]::IsNullOrEmpty($ALERTS_DIR)) {
    $ALERTS_DIR = $DEFAULT_ALERTS_DIR
    Write-Host "ALERTS_DIR: Using demo data, $(Convert-Path $ALERTS_DIR)."
}

# Function to convert Windows paths to Docker-compatible paths
function Convert-PathToDocker($path) {
    $fullPath = [System.IO.Path]::GetFullPath($path)
    $driveLetter = $fullPath.Substring(0,1).ToLower()
    $restOfPath = $fullPath.Substring(2) -replace '\\', '/'
    $dockerPath = "/$driveLetter$restOfPath"
    return $dockerPath
}

function Check-Path {
    param (
        [string]$dir_to_check,
        [string]$label
    )
    if ([string]::IsNullOrEmpty($dir_to_check)) {
        Write-Host "Checking ${label}: Error, parameter $label not set."
        exit 1
    }
    if (-not (Test-Path $dir_to_check -PathType Container)) {
        Write-Host "Checking ${label}: Error, not a valid path."
        Write-Host "${label}: $dir_to_check"
        exit 1
    }
    Write-Host "Checking ${label}:${dir_to_check} OK"
}

# Checks that ALERTS_DIR path is valid.
Check-Path $ALERTS_DIR "ALERTS_DIR"

# When used on a shared server, change BACKEND_IP with the real server IP
# as seen from the clients.
$BACKEND_IP = "127.0.0.1"

# Variables that can be changed if needed
$ALERTS_DIR_DOCKER = Convert-PathToDocker $ALERTS_DIR

$CERT_DIR = Join-Path (Get-Location).Path ".\cert"
$CERT_DIR_DOCKER = Convert-PathToDocker $CERT_DIR

# Below variables can be changed if needed, e.g. if some port is busy.
$HTTP_FRONTEND_PORT = 8080
$HTTPS_FRONTEND_PORT = 8081
$HTTP_BACKEND_PORT = 7074
$HTTPS_BACKEND_PORT = 7073

$BACKEND_REST_API = "http://${BACKEND_IP}:${HTTP_BACKEND_PORT}/api"

# Container names
$DATABASE_CONTAINER = "detect-database"
$ALERT_SENDER_CONTAINER = "detect-alert-sender"
$FRONTEND_CONTAINER = "detect-frontend"
$BACKEND_CONTAINER = "detect-backend"

$TAG = "2025.1"

# Image names
$FRONTEND_IMAGE = "percepio/detect-frontend:$TAG"
$BACKEND_IMAGE = "percepio/detect-backend:$TAG"
$ALERT_SENDER_IMAGE = "percepio/detect-alertsender:$TAG"
$DATABASE_IMAGE = "percepio/detect-database:$TAG"

# MySQL variables
$MYSQL_ROOT_PASSWORD = "DevalertMysql"
$MYSQL_DATABASE = "devalert"
$MYSQL_PORT = 3306

# Docker network
$NETWORK_NAME = "percepio-network"

# Volume name
$VOLUME_NAME = "percepio_database"

$BACKEND_ALERTS_REST_ENDPOINT = "http://${BACKEND_CONTAINER}:${HTTP_BACKEND_PORT}/api/alert"

# Function to start Docker containers
function Start-Containers {

    Write-Host "Starting Docker containers..."

    # Create a custom network if it doesn't exist
    docker network create $NETWORK_NAME | Out-Null

    # Ensure the Docker volume exists
    docker volume create $VOLUME_NAME | Out-Null

    # Start MySQL container
    docker run -d `
        --name $DATABASE_CONTAINER `
        --network $NETWORK_NAME `
        -e "MYSQL_ROOT_PASSWORD=$MYSQL_ROOT_PASSWORD" `
        -e "MYSQL_DATABASE=$MYSQL_DATABASE" `
        -p "${MYSQL_PORT}:3306" `
        -v "${VOLUME_NAME}:/var/lib/mysql" `
        $DATABASE_IMAGE

    # Start the Backend container
    docker run -d `
        --name $BACKEND_CONTAINER `
        --network $NETWORK_NAME `
        --link "${DATABASE_CONTAINER}:mysql" `
        -p "${HTTP_BACKEND_PORT}:7074" `
        -p "${HTTPS_BACKEND_PORT}:7073" `
        -e "LICENSE_KEY=$LICENSE" `
        -e "RDS_ENDPOINT=mysql" `
        -e "RDS_PORT=$MYSQL_PORT" `
        -e "RDS_DATABASE=$MYSQL_DATABASE" `
        -e "RDS_USER=root" `
        -e "RDS_PASSWORD=$MYSQL_ROOT_PASSWORD" `
        $BACKEND_IMAGE

    # Start the Frontend container
    docker run -d `
        --name $FRONTEND_CONTAINER `
        --network $NETWORK_NAME `
        -p ${HTTP_FRONTEND_PORT}:80 `
        -p ${HTTPS_FRONTEND_PORT}:443 `
        -v "${CERT_DIR_DOCKER}:/etc/nginx/ssl" `
        -e "VUE_APP_API_ENDPOINT=$BACKEND_REST_API" `
        $FRONTEND_IMAGE

    # Start the Alert Sender container
    docker run -d `
        --name $ALERT_SENDER_CONTAINER `
        --network $NETWORK_NAME `
        -e "SEND_ALERTS_URL=$BACKEND_ALERTS_REST_ENDPOINT" `
        -e "TRACE_DIR=/app/data/" `
        -e "CHECK_INTERVAL_MS=5000" `
        -v "${ALERTS_DIR_DOCKER}:/app/data" `
        --tty `
        --restart on-failure `
        $ALERT_SENDER_IMAGE

    Write-Host "Docker containers started successfully."
}

# Function to stop Docker containers and clean up
function Stop-Containers {
    Write-Host "Stopping and removing Docker containers..."

    docker stop $ALERT_SENDER_CONTAINER $FRONTEND_CONTAINER $BACKEND_CONTAINER $DATABASE_CONTAINER | Out-Null
    docker rm $ALERT_SENDER_CONTAINER $FRONTEND_CONTAINER $BACKEND_CONTAINER $DATABASE_CONTAINER | Out-Null

    Write-Host "Removing custom network..."
    docker network rm $NETWORK_NAME | Out-Null

    Write-Host "All containers stopped and removed, network '$NETWORK_NAME' deleted."
}

# Function to follow logs for all containers
function Follow-Logs {
    Write-Host "Following logs for all containers..."
    Start-Process powershell -ArgumentList "-NoExit", "-Command", "docker logs -f $DATABASE_CONTAINER"
    Start-Process powershell -ArgumentList "-NoExit", "-Command", "docker logs -f $BACKEND_CONTAINER"
    Start-Process powershell -ArgumentList "-NoExit", "-Command", "docker logs -f $FRONTEND_CONTAINER"
    Start-Process powershell -ArgumentList "-NoExit", "-Command", "docker logs -f $ALERT_SENDER_CONTAINER"
}

# Function to clean up Docker resources with a user confirmation prompt
function Cleanup-DockerResources {
   # Set the warning message color to yellow
   Write-Host -ForegroundColor Yellow "WARNING: This cleanup will delete existing alert data from the database and remove Docker containers, network, and volume."
    
   # Prompt for user confirmation
   $confirmation = Read-Host "Do you want to proceed? Type 'yes' to confirm"

    if ($confirmation -ne "yes") {
        Write-Host "Cleanup aborted by user."
        return
    }

    Write-Host "Starting full Docker cleanup..."

    # Reuse Stop-Containers to stop and remove all containers
    Stop-Containers

    # Remove Docker network if it exists
    if (docker network ls --filter name=^$NETWORK_NAME$ | Select-String $NETWORK_NAME) {
        Write-Host "Removing Docker network..."
        docker network rm $NETWORK_NAME | Out-Null
    }

    # Remove Docker volume if it exists
    if (docker volume ls --filter name=^$VOLUME_NAME$ | Select-String $VOLUME_NAME) {
        Write-Host "Removing Docker volume..."
        docker volume rm $VOLUME_NAME | Out-Null
    }

    Write-Host "Cleanup complete: Containers stopped, network and volume removed."
}


switch ($Action) {
    "start" { Start-Containers }
    "stop" { Stop-Containers }
    "logs" { Follow-Logs }
    "cleanup" { Cleanup-DockerResources }
    default {
        Write-Host "Usage: .\script.ps1 -Action {start|stop|logs}"
        exit 1
    }
}
